
#delimit;

********************************;
**********importCost************;
********************************;
/*
Generate episode-level costs

1. calculate median severity level of new convictions
2. import MCW costs
3. import UCR arrest-report ratios
4. import NCVS victim-report ratios
5. import GA crime codes with MCW crime types 
6. merge all files
7. recode severity levels and include trim
8. collapse to calculate costs by severity level
9. calculate costs and label
	where possible, assign a cost directly
    where there is no direct cost, assign the average cost of that crime's median severity level
    where no median severity level available, assign cost of severity level 1
*/

capture program drop importCost;
program importCost;

syntax, revocCost(real);

tempfile crimeMatch crimeCost sevAvgCost sevAvgCostTrim crimeReport crimeSev;

***1. calculate median severity level of new convictions;
use episodes, clear;

*obs with missing severity level are similar to level 1;
replace severitylevel = 0 if severitylevel == .;
replace severitylevel = . if severitylevel == 0;

*keep new convictions only;
egen int newConvictionDate = min(admitdate), by(uno sentdate curroff1);
keep if admitdate == newConvictionDate;

*collapse by median severity levels;
gen count = 1;
gen countSevMissing = (severitylevel == .);
collapse (median) severitylevel  (sum) count countSevMissing, by(majoroff curroff1);

*recode 8 to 7;
replace severitylevel = 7 if severitylevel == 8;
	
*some medians are halfway between two integers. round;
replace severitylevel = ceil(severitylevel);

rename curroff1 code;
sort code;
save `crimeSev', replace;

***2. import MCW costs;
insheet using Cost\NIJcostrating.csv, clear;
drop if crimetype == .;

*CPI-U inflator to 2011 dollars from 1993 dollars -- see spreadsheet in cpi folder
replace cost = cost*1.556;
di title[7] " cost is $" cost[7];
scalar rapecost = cost[7];

keep crimetype cost title;
sort crimetype;
save `crimeCost', replace;

***3. import UCR report-arrest ratios;
*import reports;
use UCR/1996C, clear;

local varlist = "INDEX MURDER RAPE ROBBERY AGASSLT BURGLRY LARCENY MVTHEFT ARSON";

foreach var of varlist `varlist' {;
	rename `var' report`var';
};

collapse (sum) report* if FIPS_ST == 13, by(FIPS_ST);
list;
sort FIPS_ST;
tempfile report96;
save `report96';

*import arrests;
use UCR/1996A, clear;
gen INDEX = MURDER+RAPE+ROBBERY+AGASSLT+BURGLRY+LARCENY+MVTHEFT;
collapse (sum) `varlist' if FIPS_ST == 13, by(FIPS_ST);
list;
sort FIPS_ST;
merge FIPS_ST using `report96', unique;
tab _merge;
drop _merge;


foreach var of varlist `varlist' {;
	gen reportRatio`var' = report`var'/`var';
};
keep reportRatio*;

gen x = _n;
reshape long reportRatio, i(x) j(crime) string;
drop x;
gen byte crimetype = 9 if crime == "AGASSLT";
replace crimetype = 13 if crime == "ARSON";
replace crimetype = 15 if crime == "BURGLRY";
replace crimetype = 14 if crime == "LARCENY";
replace crimetype = 1 if crime == "MURDER";
replace crimetype = 16 if crime == "MVTHEFT";
replace crimetype = 8 if crime == "RAPE";
replace crimetype = 11 if crime == "ROBBERY";

list;
di "Report-arrest default ratio is " reportRatio[4];
scalar reportArrestDefault = reportRatio[4];
drop if crimetype == .;

sort crimetype;
tempfile ucr;
save `ucr';

***4. import NCVS victim-report ratios;
*Source: http://bjs.ojp.usdoj.gov/content/pub/sheets/cv93c6.txt;
insheet using Cost/crimereport.csv, names comma clear case;
keep if crimetype != .;

*check that I got alignment right for all columns;
assert checksum == "100";
keep crimetype victimReport;
destring crimetype victimReport, replace;

replace victimReport = 100/victimReport;
sort crimetype;

di "Victim-report default ratio is " victimReport[1];
scalar victimReportDefault = victimReport[1];
drop if crimetype == 0;
sort crimetype;
tempfile ncvs;
save `ncvs';

***5. import GA crime codes with MCW crime types;
insheet using Cost\crimesTimCarr.csv, clear comma names;

label define crimetype 1 "Fatal Assault/Rape",add;
label define crimetype 3 "Fatal DWI",add;
label define crimetype 5 "Child Sexual Abuse",add;
label define crimetype 6 "Child Physical Abuse",add;
label define crimetype 8 "Rape/Sexual Assault",add;
label define crimetype 9 "Other Assault",add;
label define crimetype 10 "Domestic",add;
label define crimetype 11 "Robbery",add;
label define crimetype 12 "Drunk Driving",add;
label define crimetype 13 "Arson",add;
label define crimetype 14 "Larceny",add;
label define crimetype 15 "Burglary",add;
label define crimetype 16 "Motor Vehicle Theft",add;
label define crimetype 17 "Child Neglect",add;
label define crimetype -1 "Revocation - Felony",add;
label define crimetype -2 "Revocation - Misdemeanor",add;
label define crimetype -3 "Revocation - Techincal",add;
label define crimetype -11 "Severity Level 1",add;
label define crimetype -12 "Severity Level 2",add;
label define crimetype -13 "Severity Level 3",add;
label define crimetype -14 "Severity Level 4",add;
label define crimetype -15 "Severity Level 5",add;
label define crimetype -16 "Severity Level 6",add;
label define crimetype -17 "Severity Level 7",add;

label values crimetype crimetype;

rename offns_code code;
keep code crimetype offns_name;
sort code;
tempfile crimesTimCarr;
save `crimesTimCarr';

***6. merge all files;
*take offender records with parole ratings since 1990 with new convictions
from first merge, there are four crime codes which do not appear in the list
that Tim Carr sent me. All four are missing severity codes. These get folded into
the other offenses which are missing severity codes.;
use `crimeSev', clear;

*merge crime type;
sort code;
merge code using `crimesTimCarr', unique;
tab _merge;
rename _merge _mergeTimCarr;
gen codenum = code;

*merge NCVS crime cost where available;
sort crimetype;
merge crimetype using `crimeCost', uniqusing;
tab _merge;
drop if _merge == 2;
rename _merge costDirect;

*merge NCVS victimization to report ratio where available;
sort crimetype;
merge crimetype using `ncvs', uniqusing;
tab _merge;
*1 = need to infer cost, 3 = we have cost directly.;
rename _merge mergeVRratio;

*merge UCR report to arrest ratio where available;
sort crimetype;
merge crimetype using `ucr', uniqusing;
tab _merge;
rename _merge mergeUCR;

***7. recode severity levels and include trim
	murder to 7
	sexual assault to 5
	dui to missing;

*recode murder to severitylevel 7. this should define the upper 
end of the distribution and ignore intent (which put invol manslaughter at 5
and vol manslaughter at 6);
replace severitylevel = 7 if crimetype <= 3 & severitylevel != .;

*recode sexual assault between a custodian and person in
custody and sexual battery to severity level 5. for comparison,
 enticing a child and aggravated sexual battery are level 5;
replace severitylevel = 5 if severitylevel < 5 & 
	(crimetype == 5 | crimetype == 8);
	
*recode DUI's b/c they appear nowhere in the sev table;
replace severitylevel = . if crimetype == 12;

*create a version of the table with trimmed murder costs and zero drug costs;
gen costTrim = cost;
replace costTrim = rapecost*2 if crimetype <= 3;
replace costTrim = 0 if majoroff == 4 | majoroff == 5;

***8. collapse to calculate costs by severity level;
*recode missing severity level to 2 -- only 1% of crimes have a missing median severity level;
preserve;
collapse costSev=cost costSevTrim=costTrim [fweight=count], by(severitylevel);
drop if severitylevel == .;
sort severitylevel;
save `sevAvgCost', replace;
restore;

gen noSevAvail = (count == countSevMissing);
replace noSevAvail = 2 if countSevMissing + 10 < count & count != countSevMissing;
assert costDirect != .;
replace severitylevel = 1 if severitylevel == . & costDirect == 1;

sort severitylevel;
merge severitylevel using `sevAvgCost', uniqusing;
*only misses are obs who already have cost directly;
tab _merge;
tab costDirect if _merge == 1;
assert cost != . if costDirect == 3;
assert costSev != . if costDirect == 1;
drop _merge;



***9. calculate costs 
	assign parole revocation cost directly
	where there is direct cost, assign a cost directly
    where there is no direct cost, assign the average cost of that crime's median severity level;
    
*incorporate defaults for parole revocations and missing crime codes
0 means charge is missing or non-existent (see curroff10, where most obs
are 0) and so set cost there to zero;
count;
local newcount = r(N) + 3;
set obs `newcount';
replace code = -1 if _n == `newcount'-2;
replace code = -2 if _n == `newcount'-1;
replace code = -3 if _n == `newcount';
replace cost = `revocCost' if code == -1;
replace cost = `revocCost' if code == -2;
replace cost = `revocCost' if code == -3;
replace cost = 0 if code == 0;
replace costDirect = 2 if code < 0;


*sub in default values for victim-report and report-arrest ratios;
replace victimReport = victimReportDefault if victimReport == .;
replace victimReport = 1 if crimetype <=3;
replace reportRatio = reportArrestDefault if reportRatio == .; 

*cost1;
*costDirect=1 is cost missing, 2 is parole revoc, 3 is match;
replace cost = costSev if costDirect == 1;

*cost2 - zero drugs, murder = 2*rape, use lower table values;
gen costLow = 0 if majoroff == 4 | majoroff == 5;
replace costLow = rapecost*2 if crimetype <= 3;
replace costLow = cost if costLow == .;
replace costLow = costSevTrim if costDirect == 1 & costLow != 0 & crimetype > 3;

*cost3, cost4, cost5;
gen costHigh = cost*victimReport*reportRatio;
gen costHighTrim = costLow*victimReport*reportRatio;
gen costHighPref = costLow*reportRatio;

assert cost != .;
assert costLow != .;
assert costHigh != .;
assert costHighTrim != .;
assert costHighPref != .;

rename severitylevel severityCode;

keep code cost* severity crimetype majoroff;
rename code curroff1;
sort curroff1;

save crimelist, replace;

***numbers for cost appendix;
tab severityCode if costDirect == 3, sum(cost);

end;

********************************;
**********assignCost************;
********************************;
/*
Calculates cost of most serious offense, for 5 different cost specifications.

1. Figure out if episode is for a new crime or a revocation. Recode revocations.
2. Merge costs, see summary stats
3. Calculate forward-looking costs
*/

capture program drop assignCost;
program assignCost;
syntax, horizon(int) filename(string) crimelist(string);

use `filename', clear;



***2. Merge costs, see summary stats;

sort curroff1;
merge curroff1 using `crimelist', uniqusing;
tab _merge;

gen cost1 = cost;
gen cost2 = costLow;
gen cost3 = costHigh;
gen cost4 = costHighTrim;
gen cost5 = costHighPref;

*recode crimetype var to cover universe of crime costs (parole revoc same, -sev level-10);
replace crimetype = curroff1 if curroff1 < 0;
replace crimetype = -10-severityCode if costDirect == 1;

*summary stats on costs for each prison episode;
qui count if cost1 == .;
di r(N) " obs are missing a cost.";
sum cost*;


*generate summary stats for appendix;
*What frac of costs and what frac of people are coming from direct merges?;
egen byte inSamp = max(baseObserv), by(uno);
gen withinHorizon10 = (inSamp & !baseObserv & admitdate < baseRelDate + 365*10
	& admitdate > baseAdmitDate);
table costDirect if withinHorizon10, c(freq sum cost1) row col;


***3. Calculate forward-looking costs;
foreach t of numlist 1 3 10 {;
	cap drop withinHorizon withinHorizonIncap;
	gen byte withinHorizon = (!baseObserv & admitdate < baseRelDate + 365*`t'	& admitdate > baseAdmitDate);
	gen byte withinHorizonIncap = (!baseObserv & admitdate < baseRelDate + 365*(`t'+1) & admitdate > baseAdmitDate);

	*calc costs;
	egen float fCost5_`t' = sum(cost5*withinHorizon), by(uno);
	
	*count up event types;	
	egen byte fEpisodeCount_`t' = sum(withinHorizon), by(uno);
	egen byte fEpisodeCount_Start`t' = sum(withinHorizonIncap), by(uno);
	replace fEpisodeCount_Start`t' =  fEpisodeCount_`t' if after;
	egen byte fRevocEpisodeCount_`t' = sum(withinHorizon*revoc), by(uno);
	egen byte fCostEpisodeCount_`t' = sum(withinHorizon*!revoc), by(uno);
};

forvalues x = 1/4 {;
egen float fCost`x'_10 = sum(cost`x'*withinHorizon), by(uno);
};



compress;

end;

